@@ -243,6 +243,17 @@
              (gc (find-if (lambda (x) (member x '(:cheneygc :gencgc)))
                           target-feature-list))
              (arch (target-platform-keyword target-feature-list)))
+        (when (and (member :x86 target-feature-list)
+                   (member :int4-breakpoints target-feature-list))
+          ;; 0xCE is a perfectly good 32-bit instruction,
+          ;; unlike on x86-64 where it is illegal. It's therefore
+          ;; confusing to allow this feature in a 32-bit build.
+          ;; But it's annoying to have a build script that otherwise works
+          ;; for a native x86/x86-64 build except for needing one change.
+          ;; Just print something and go on with life.
+          (setq target-feature-list
+                (remove :int4-breakpoints target-feature-list))
+          (warn "Removed :INT4-BREAKPOINTS from target features"))
         ;; Putting arch and gc choice first is visually convenient, versus
         ;; having to parse a random place in the line to figure out the value
         ;; of a binary choice {cheney vs gencgc} and architecture.
@@ -309,11 +320,6 @@
           ":IMMOBILE-CODE requires :IMMOBILE-SPACE feature")
          ("(and immobile-symbols (not immobile-space))"
           ":IMMOBILE-SYMBOLS requires :IMMOBILE-SPACE feature")
-         ("(and int4-breakpoints x86)"
-          ;; 0xCE is a perfectly good 32-bit instruction,
-          ;; unlike on x86-64 where it is illegal. It's therefore
-          ;; confusing to allow this feature in a 32-bit build.
-          ":INT4-BREAKPOINTS are incompatible with x86")
          ;; There is still hope to make multithreading on DragonFly x86-64
          ("(and sb-thread x86 dragonfly)"
           ":SB-THREAD not supported on selected architecture")))
@@ -396,18 +402,12 @@
     ;; just to avoid that ugly mess.
     nil))
 
-;;; The specialized array registry has file-wide scope. Hacking that aspect
-;;; into the xc build scaffold seemed slightly easier than hacking the
-;;; compiler (i.e. making the registry a slot of the fasl-output struct)
-(defvar *array-to-specialization* (make-hash-table :test #'eq))
-
 (defmacro do-stems-and-flags ((stem flags build-phase) &body body)
   (let ((stem-and-flags (gensym "STEM-AND-FLAGS")))
     `(dolist (,stem-and-flags (get-stems-and-flags ,build-phase))
        (let ((,stem (first ,stem-and-flags))
              (,flags (rest ,stem-and-flags)))
-         ,@body
-         (clrhash *array-to-specialization*)))))
+         ,@body))))
 
 ;;; Given a STEM, remap the path component "/target/" to a suitable
 ;;; target directory.
@@ -534,7 +534,7 @@
                                               *target-assemble-file*
                                               *target-compile-file*))))
          (trace-file (if (find :trace-file flags) t nil))
-         (block-compile (if (find :block-compile flags) t nil))
+         (block-compile (if (find :block-compile flags) t :specified))
          (ignore-failure-p (find :ignore-failure-p flags)))
     (declare (type function compile-file))
 
@@ -586,12 +586,8 @@
        retry-compile-file
          (multiple-value-bind (output-truename warnings-p failure-p)
              (restart-case
-                 (funcall compile-file src :output-file tmp-obj
-                          ;; if tracing, also show high-level progress
-                          :trace-file trace-file
-                          :print trace-file
-                          :block-compile (and block-compile
-                                              ;; Block compilation was
+                 (apply compile-file src :output-file tmp-obj
+                          :block-compile (and ;; Block compilation was
                                               ;; completely broken
                                               ;; from the beginning of
                                               ;; SBCL history until
@@ -602,8 +598,14 @@
                                                        (funcall (find-symbol "VERSION>=" "SB-C")
                                                                 (funcall (find-symbol "SPLIT-VERSION-STRING" "SB-C")
                                                                          (lisp-implementation-version))
-                                                                '(2 0 2)))))
-                          :allow-other-keys t)
+                                                                '(2 0 2))))
+                                              block-compile)
+                          :allow-other-keys t
+                          ;; If tracing, also print, but don't specify :PRINT unless specifying
+                          ;; :TRACE-FILE so that whatever the default is for *COMPILE-PRINT*
+                          ;; prevails, insensitively to whether it's the SB-XC: or CL: symbol.
+                          (when trace-file
+                            '(:trace-file t :print t)))
                (recompile ()
                  :report report-recompile-restart
                  (go retry-compile-file)))
