@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (define-move-fun (load-immediate 1) (vop x y)
   ((null zero immediate)
@@ -83,8 +83,6 @@
             :load-if (not (location= x y))))
   (:results (y :scs (any-reg descriptor-reg control-stack)
                :load-if (not (location= x y))))
-  (:effects)
-  (:affected)
   (:generator 0
     (unless (location= x y)
       (sc-case y
@@ -97,11 +95,6 @@
   (any-reg descriptor-reg zero null)
   (any-reg descriptor-reg))
 
-;;; Make MOVE the check VOP for T so that type check generation
-;;; doesn't think it is a hairy type. This also allows checking of a
-;;; few of the values in a continuation to fall out.
-(primitive-type-vop move (:check) t)
-
 ;;; The MOVE-ARG VOP is used for moving descriptor values into
 ;;; another frame for argument or known value passing.
 (define-vop (move-arg)
@@ -121,21 +114,6 @@
   (any-reg descriptor-reg null zero)
   (any-reg descriptor-reg))
 
-;;;; ILLEGAL-MOVE
-
-;;; This VOP exists just to begin the lifetime of a TN that couldn't
-;;; be written legally due to a type error. An error is signalled
-;;; before this VOP is so we don't need to do anything (not that there
-;;; would be anything sensible to do anyway.)
-(define-vop (illegal-move)
-  (:args (x) (type))
-  (:results (y))
-  (:ignore y)
-  (:vop-var vop)
-  (:save-p :compute-only)
-  (:generator 666
-    (error-call vop object-not-type-error x type)))
-
 ;;;; moves and coercions
 ;;;;
 ;;;; These MOVE-TO-WORD VOPs move a tagged integer to a raw full-word
@@ -161,7 +139,7 @@
   (:results (y :scs (signed-reg unsigned-reg)))
   (:note "constant load")
   (:generator 1
-    (cond ((sb!c::tn-leaf x)
+    (cond ((sb-c::tn-leaf x)
            (inst li (tn-value x) y))
           (t
            (loadw y code-tn (tn-offset x) other-pointer-lowtag)
@@ -244,6 +222,32 @@
 (define-move-vop move-from-signed :move
   (signed-reg) (descriptor-reg))
 
+(define-vop (move-from-fixnum+1)
+  (:args (x :scs (signed-reg unsigned-reg)))
+  (:results (y :scs (any-reg descriptor-reg)))
+  (:temporary (:scs (non-descriptor-reg)) temp)
+  (:vop-var vop)
+  (:generator 4
+    (inst sra x n-positive-fixnum-bits temp)
+    (inst sll x n-fixnum-tag-bits y)
+    (inst beq temp done)
+    (inst not temp temp)
+    (inst beq temp done)
+    (load-constant vop (emit-constant (1+ sb-xc:most-positive-fixnum))
+                   y)
+    DONE))
+
+(define-vop (move-from-fixnum-1 move-from-fixnum+1)
+  (:generator 4
+    (inst sra x n-positive-fixnum-bits temp)
+    (inst sll x n-fixnum-tag-bits y)
+    (inst beq temp done)
+    (inst not temp temp)
+    (inst beq temp done)
+    (load-constant vop (emit-constant (1- sb-xc:most-negative-fixnum))
+                   y)
+    DONE))
+
 ;;; Check for fixnum, and possibly allocate one or two word bignum
 ;;; result. Use a worst-case cost to make sure people know they may be
 ;;; number consing.
@@ -283,8 +287,6 @@
             :load-if (not (location= x y))))
   (:results (y :scs (signed-reg unsigned-reg)
                :load-if (not (location= x y))))
-  (:effects)
-  (:affected)
   (:note "word integer move")
   (:generator 0
     (move x y)))
