@@ -873,9 +873,19 @@
 
 (defknown list (&rest t) list (movable flushable))
 (defknown list* (t &rest t) t (movable flushable))
-(defknown make-list (index &key (:initial-element t)) list
+;;; The length constraint on MAKE-LIST is such that:
+;;; - not every byte of addressable memory can be used up.
+;;; - the number of bytes to allocate should not be a bignum,
+;;    nor can its native representation be larger than 'sword_t'
+;;; So it's half of most-positive-word divided by the cons size,
+;;; which is roughly twice as small as array-dimension-limit on 32-bit machines,
+;;; or 8x smaller on 64-bit machines.
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defconstant make-list-limit
+    (floor (/ sb-ext:most-positive-word 2) (* 2 sb-vm:n-word-bytes))))
+(defknown make-list ((integer 0 (#.make-list-limit)) &key (:initial-element t)) list
   (movable flushable))
-(defknown %make-list (index t) list (movable flushable))
+(defknown %make-list ((integer 0 (#.make-list-limit)) t) list (movable flushable))
 
 (defknown sb-impl::|List| (&rest t) list (movable flushable foldable))
 (defknown sb-impl::|List*| (t &rest t) t (movable flushable foldable))
@@ -1041,6 +1051,7 @@
 (defknown hash-table-test (hash-table) symbol (foldable flushable))
 (defknown (sxhash psxhash) (t) hash-code (foldable flushable))
 (defknown hash-table-equalp (hash-table hash-table) boolean (foldable flushable))
+(defknown sb-impl::install-hash-table-lock (hash-table) sb-thread:mutex ())
 ;; To avoid emitting code to test for nil-function-returned
 (defknown (sb-impl::signal-corrupt-hash-table
            sb-impl::signal-corrupt-hash-table-bucket)
@@ -1250,12 +1261,14 @@
 (defknown sb-impl::%init-string-input-stream (instance string &optional index sequence-end)
   (values sb-impl::string-input-stream index &optional)
   (flushable))
+(defknown sb-impl::%init-string-output-stream (instance t t) sb-impl::string-output-stream
+  ;; Not flushable, for two reasons:
+  ;; - possibly need to verify that the second arg is a type specifier
+  ;; - if you don't initialize the stream, then GET-OUTPUT-STRING-STREAM will crash
+  ())
 (defknown make-string-output-stream (&key (:element-type type-specifier))
   sb-impl::string-output-stream
   (flushable))
-;; FIXME: sb-impl::string-output-stream as the result type causes a few
-;; "CROSS-TYPEP uncertain: CTYPEP T #<UNKNOWN-TYPE STRING-OUTPUT-STREAM>"
-;; warnings which need to be resolved.
 (defknown get-output-stream-string (stream) simple-string ())
 (defknown streamp (t) boolean (movable foldable flushable))
 (defknown stream-element-type (stream) type-specifier ; can it return a CLASS?
@@ -2045,7 +2058,7 @@
 ;;; by wiring in the needed functions instead of dereferencing their fdefns.
 (defknown (ill-in ill-bin ill-out ill-bout
            sb-impl::string-inch sb-impl::string-in-misc
-           sb-impl::string-ouch sb-impl::string-sout sb-impl::string-out-misc
+           sb-impl::string-sout
            sb-impl::finite-base-string-ouch sb-impl::finite-base-string-out-misc
            sb-impl::fill-pointer-ouch sb-impl::fill-pointer-sout
            sb-impl::fill-pointer-misc
