@@ -52,6 +52,7 @@
   ;; which is used for a fixed #\NULL so that when we call out to C
   ;; we don't need to cons a new copy)
   (n-pad-elements (missing-arg) :type index :read-only t))
+(declaim (freeze-type specialized-array-element-type-properties))
 
 (define-load-time-global *specialized-array-element-type-properties*
   (map 'simple-vector
@@ -77,10 +78,9 @@
                     :complex-typecode #.complex-base-string-widetag)
          #+sb-unicode
          (character ,(code-char 0) 32 simple-character-string
-                    :n-pad-elements 1
                     :complex-typecode #.complex-character-string-widetag)
-         (single-float 0.0f0 32 simple-array-single-float)
-         (double-float 0.0d0 64 simple-array-double-float)
+         (single-float $0.0f0 32 simple-array-single-float)
+         (double-float $0.0d0 64 simple-array-double-float)
          (bit 0 1 simple-bit-vector
               :complex-typecode #.complex-bit-vector-widetag)
          ;; KLUDGE: The fact that these UNSIGNED-BYTE entries come
@@ -125,12 +125,12 @@
          (fixnum 0 64 simple-array-fixnum :fixnum-p t)
          #+64-bit
          ((signed-byte 64) 0 64 simple-array-signed-byte-64)
-         ((complex single-float) #C(0.0f0 0.0f0) 64
+         ((complex single-float) ,(complex $0f0 $0f0) 64
           simple-array-complex-single-float)
-         ((complex double-float) #C(0.0d0 0.0d0) 128
+         ((complex double-float) ,(complex $0d0 $0d0) 128
           simple-array-complex-double-float)
          #+long-float
-         ((complex long-float) #C(0.0l0 0.0l0) #+x86 192 #+sparc 256
+         ((complex long-float) ,(complex $0L0 $0L0) #+x86 192 #+sparc 256
           simple-array-complex-long-float)
          (t 0 #.n-word-bits simple-vector))))
 
@@ -156,16 +156,6 @@
        ;; than the word size.
        (<= (saetp-n-bits saetp) n-word-bits)))
 
-#+sb-xc-host
-(defvar sb-kernel::*specialized-array-element-types*
-  (map 'list
-       #'saetp-specifier
-       *specialized-array-element-type-properties*))
-
-#-sb-xc-host
-(!define-load-time-global sb-kernel::*specialized-array-element-types*
-            '#.sb-kernel::*specialized-array-element-types*)
-
 (define-load-time-global *vector-without-complex-typecode-infos*
   #+sb-xc-host
   (loop for saetp across *specialized-array-element-type-properties*
