@@ -34,7 +34,6 @@
 
 (in-package "SB-IMPL")
 
-#-no-ansi-print-object
 (defmethod print-object ((x meta-info) stream)
   (print-unreadable-object (x stream)
     (format stream "~S ~S, ~D" (meta-info-category x) (meta-info-kind x)
@@ -79,17 +78,15 @@
         ;; The SB-FASL: symbols are poor style, but the lesser evil.
         ;; If exported, then they'll stick around in the target image.
         ;; Perhaps SB-COLD should re-export some of these.
-        (declare (special sb-fasl::*dynamic* sb-fasl::*cold-layouts*))
-        (let ((layout (gethash 'meta-info sb-fasl::*cold-layouts*)))
-          (sb-fasl::cold-svset
-           (sb-fasl::cold-symbol-value '*info-types*)
-           id
-           (sb-fasl::write-slots
-            (sb-fasl::allocate-struct sb-fasl::*dynamic* layout)
-            'meta-info ; give the type name in lieu of layout
-            :category category :kind kind :type-spec type-spec
-            :type-checker checker :validate-function validator
-            :default default :number id)))))
+        (sb-fasl::cold-svset
+         (sb-fasl::cold-symbol-value '*info-types*)
+         id
+         (sb-fasl::write-slots
+          (sb-fasl::allocate-struct-of-type 'meta-info)
+          'meta-info ; pass the type name in lieu of layout
+          :category category :kind kind :type-spec type-spec
+          :type-checker checker :validate-function validator
+          :default default :number id))))
 
 ;;;; info types, and type numbers, part II: what's
 ;;;; needed only at compile time, not at run time
@@ -189,7 +186,7 @@
                   (if old
                       ;; if -REMOVE => nil, then update NEW but return OLD
                       (or (setq new (packed-info-remove
-                                     old +no-auxilliary-key+ info-numbers))
+                                     old +no-auxiliary-key+ info-numbers))
                           old))))
         (info-puthash *info-environment* name #'clear-hairy)))
     (not (null new))))
@@ -237,7 +234,7 @@
            ;; but constraint propagation isn't informing the compiler here.
            :simple (values (symbol-info-vector (truly-the symbol key1)) key2)
            :hairy (values (info-gethash name *info-environment*)
-                          +no-auxilliary-key+)))
+                          +no-auxiliary-key+)))
       (when vector
         (let ((index (packed-info-value-index vector aux-key info-number)))
           (when index
@@ -281,8 +278,9 @@
 (define-info-type (:function :deprecated)
   :type-spec (or null deprecation-info))
 
-(declaim (ftype (sfunction (t) ctype)
-                specifier-type ctype-of sb-kernel::ctype-of-array))
+;;; Why are these here? It seems like the wrong place.
+(declaim (ftype (sfunction (t &optional t symbol) ctype) specifier-type)
+         (ftype (sfunction (t) ctype) ctype-of sb-kernel::ctype-of-array))
 
 ;;; the ASSUMED-TYPE for this function, if we have to infer the type
 ;;; due to not having a declaration or definition
@@ -366,6 +364,9 @@
 ;;; structure containing the info used to special-case compilation.
 (define-info-type (:function :info) :type-spec (or sb-c::fun-info null))
 
+;;; For PCL code walker
+(define-info-type (:function :walker-template) :type-spec (or list symbol))
+
 ;;; This is a type specifier <t> such that if an argument X to the function
 ;;; does not satisfy (TYPEP x <t>) then the function definitely returns NIL.
 ;;; When the named function is a predicate that appears in (SATISFIES p)
@@ -450,7 +451,7 @@
                        ;; The compiler-macro signals an error
                        ;; on forward-referenced info-types.
                        #+sb-xc-host (declare (notinline info))
-                       (when (info :declaration :recognized name)
+                       (when (info :declaration :known name)
                          (error 'declaration-type-conflict-error
                                 :format-arguments (list name)))))
 
@@ -492,8 +493,8 @@
 ;; Therefore maintain a list of recognized declarations. This list makes the
 ;; globaldb storage of same redundant, but oh well.
 (defglobal *recognized-declarations* nil)
-(define-info-type (:declaration :recognized)
-  :type-spec boolean
+(define-info-type (:declaration :known)
+  :type-spec (or function boolean)
   ;; There's no portable way to unproclaim that a symbol is a declaration,
   ;; but at the low-level permit new-value to be NIL.
   :validate-function (lambda (name new-value)
@@ -507,8 +508,6 @@
                               (setq *recognized-declarations*
                                     (delete name *recognized-declarations*))))))
 
-(define-info-type (:declaration :handler) :type-spec (or function null))
-
 ;;;; ":ALIEN-TYPE" subsection - Data pertaining to globally known alien-types.
 (define-info-type (:alien-type :kind)
   :type-spec (member :primitive :defined :unknown)
