@@ -43,33 +43,33 @@
   (cache nil :type (or cache null))
   (pv-size 0 :type fixnum)
   (slot-name-lists nil :type list))
+(declaim (freeze-type pv-table))
 
 (defun make-pv-table-type-declaration (var)
   `(type pv-table ,var))
 
 ;;; Used for interning parts of SLOT-NAME-LISTS, as part of
 ;;; PV-TABLE interning -- just to save space.
-(defvar *slot-name-lists* (make-hash-table :test 'equal))
+(define-load-time-global *slot-name-lists* (make-hash-table :test 'equal))
 
 ;;; Used for interning PV-TABLES, keyed by the SLOT-NAME-LISTS
 ;;; used.
-(defvar *pv-tables* (make-hash-table :test 'equal))
+(define-load-time-global *pv-tables* (make-hash-table :test 'equal))
 
 ;;; ...and one lock to rule them. Lock because for certain (rare)
 ;;; cases this lock might be grabbed in the course of method dispatch
 ;;; -- and mostly this is already under the *world-lock*
-(defvar *pv-lock*
+(define-load-time-global *pv-lock*
   (sb-thread:make-mutex :name "pv table index lock"))
 
 (defun intern-pv-table (&key slot-name-lists)
   (flet ((intern-slot-names (slot-names)
-           (or (gethash slot-names *slot-name-lists*)
-               (setf (gethash slot-names *slot-name-lists*) slot-names)))
+           (ensure-gethash slot-names *slot-name-lists* slot-names))
          (%intern-pv-table (snl)
-           (or (gethash snl *pv-tables*)
-               (setf (gethash snl *pv-tables*)
-                     (make-pv-table :slot-name-lists snl
-                                    :pv-size (* 2 (reduce #'+ snl :key #'length)))))))
+           (ensure-gethash
+            snl *pv-tables*
+            (make-pv-table :slot-name-lists snl
+                           :pv-size (* 2 (reduce #'+ snl :key #'length))))))
     (sb-thread:with-mutex (*pv-lock*)
       (%intern-pv-table (mapcar #'intern-slot-names slot-name-lists)))))
 
@@ -96,7 +96,7 @@
       (dolist (slot-names slot-name-lists)
         (when slot-names
           (let* ((wrapper (pop wrappers))
-                 (std-p (layout-for-std-class-p wrapper))
+                 (std-p (layout-for-pcl-obj-p wrapper))
                  (class (wrapper-class* wrapper)))
             (dolist (slot-name slot-names)
               (destructuring-bind (location . info)
@@ -167,11 +167,10 @@
                                                        parameter-or-nil
                                                        env)))
                    (class (find-class class-name nil)))
-              (cond ((not (eq **boot-state** 'complete))
-                     (setq class nil))
+              (cond ((or (not (eq **boot-state** 'complete))
+                         (forward-referenced-class-p class))
+                     (setf class nil))
                     ((and class (not (class-finalized-p class)))
-                     ;; The class itself is never forward-referenced
-                     ;; here, but its superclasses may be.
                      (unless (try-finalize-inheritance class)
                        (when (boundp 'sb-c:*lexenv*)
                          (sb-c:compiler-notify
@@ -768,8 +767,8 @@
                             ,@inner-decls
                             ,@body-sans-decls))))
                  (mf (%make-method-function fmf)))
-            (set-funcallable-instance-function
-             mf (method-function-from-fast-function fmf ',(getf initargs 'plist)))
+            (setf (%funcallable-instance-fun mf)
+                  (method-function-from-fast-function fmf ',(getf initargs 'plist)))
             mf)
           ',initargs)))))
 
